/*!\file: thermal_core.cpp
 * \brief: core of the thermal solution 
 */ 

#undef __FUNCT__ 
#define __FUNCT__ "thermal_core"

#include "../toolkits/toolkits.h"
#include "../objects/objects.h"
#include "../shared/shared.h"
#include "../EnumDefinitions/EnumDefinitions.h"
#include "./parallel.h"
#include "../issm.h"

void thermal_core(DataSet* results,FemModel* fems, ParameterInputs* inputs){

	extern int my_rank;
	int i;

	/*fem models: */
	FemModel* fem_t=NULL;
	FemModel* fem_m=NULL;

	/*output: */
	Result* result=NULL;

	/*solutions vectors: */
	Vec*    t_g=NULL;
	Vec*    m_g=NULL;
	double* time=NULL;

	/*flags: */
	int     debug=0;
	int     numberofdofspernode;
	int     numberofnodes;
	int     nsteps;
	double  dt;
	double  ndt;

	int    sub_analysis_type;
	double melting_offset;
	
	Param*           param=NULL;

	/*recover fem models: */
	fem_t=fems+0;
	fem_m=fems+1;

	//first recover parameters common to all solutions
	fem_t->parameters->FindParam((void*)&numberofnodes,"numberofnodes");
	fem_t->parameters->FindParam((void*)&sub_analysis_type,"sub_analysis_type");
	fem_t->parameters->FindParam((void*)&debug,"debug");
	fem_t->parameters->FindParam((void*)&dt,"dt");
	fem_t->parameters->FindParam((void*)&ndt,"ndt");

	if(sub_analysis_type==SteadyAnalysisEnum()){

		time=(double*)xmalloc(sizeof(double));
		time[0]=0;

		/*allocate t_g and m_g arrays: */
		t_g=(Vec*)xmalloc(sizeof(Vec));
		m_g=(Vec*)xmalloc(sizeof(Vec));

		if(debug)_printf_("computing temperatures:\n");
		thermal_core_nonlinear(&t_g[0],&melting_offset,fem_t,inputs,ThermalAnalysisEnum(),SteadyAnalysisEnum());
		inputs->Add("temperature",t_g[0],1,numberofnodes);
		inputs->Add("melting_offset",melting_offset);
		
		if(debug)_printf_("computing melting:\n");
		diagnostic_core_linear(&m_g[0],fem_m,inputs,MeltingAnalysisEnum(),SteadyAnalysisEnum());
	}
	else{
		
		nsteps=(int)(ndt/dt);
		time=(double*)xmalloc((nsteps+1)*sizeof(double));
		time[0]=0;

		/*allocate t_g and m_g arrays: */
		t_g=(Vec*)xmalloc((nsteps+1)*sizeof(Vec));
		m_g=(Vec*)xmalloc((nsteps+1)*sizeof(Vec));

		//initialize temperature and melting
		fem_t->parameters->FindParam((void*)&t_g[0],"t_g");
		fem_m->parameters->FindParam((void*)&m_g[0],"m_g");

		//erase temperature and melting embedded in parameters
		param=(Param*)fem_t->parameters->FindParamObject("t_g");
		fem_t->parameters->DeleteObject((Object*)param);
		param=(Param*)fem_m->parameters->FindParamObject("m_g");
		fem_m->parameters->DeleteObject((Object*)param);

		for(i=0;i<nsteps;i++){
			if(debug)_printf_("time step: %i/%i\n",i+1,nsteps);
			time[i+1]=(i+1)*dt;
			
			if(debug)_printf_("computing temperatures:\n");
			inputs->Add("temperature",t_g[i],1,numberofnodes);
			thermal_core_nonlinear(&t_g[i+1],&melting_offset,fem_t,inputs,ThermalAnalysisEnum(),TransientAnalysisEnum());
			
			if(debug)_printf_("computing melting:\n");
			inputs->Add("temperature",t_g[i+1],1,numberofnodes);
			inputs->Add("melting_offset",melting_offset);
			diagnostic_core_linear(&m_g[i+1],fem_m,inputs,MeltingAnalysisEnum(),TransientAnalysisEnum());
		}
	}
	
	/*Plug results into output dataset: */
	if(sub_analysis_type==SteadyAnalysisEnum()){
		result=new Result(results->Size()+1,0,1,"t_g",t_g[0]);
		results->AddObject(result);
		
		result=new Result(results->Size()+1,0,1,"m_g",m_g[0]);
		results->AddObject(result);
	}
	else{
		for(i=0;i<nsteps+1;i++){
			result=new Result(results->Size()+1,time[i],i+1,"t_g",t_g[i]);
			results->AddObject(result);

			result=new Result(results->Size()+1,time[i],i+1,"m_g",m_g[i]);
			results->AddObject(result);
		}
	}

}
